@@ -24,6 +24,7 @@
 #include "qgssymbol.h"
 #include "qgsfields.h"
 #include "qgsfeaturerequest.h"
+#include "qgssymbollayerreference.h"
 
 #include <QList>
 #include <QString>
@@ -63,8 +64,18 @@ class CORE_EXPORT QgsSymbolLevelItem
       : mSymbol( symbol )
       , mLayer( layer )
     {}
-    QgsSymbol *symbol() { return mSymbol; }
-    int layer() { return mLayer; }
+
+    /**
+     * The symbol of this symbol level
+     */
+    QgsSymbol *symbol() const;
+
+    /**
+     * The layer of this symbol level
+     */
+    int layer() const;
+
+    // TODO QGIS 4.0 -> make private
   protected:
     QgsSymbol *mSymbol = nullptr;
     int mLayer;
@@ -96,25 +107,25 @@ class CORE_EXPORT QgsFeatureRenderer
 
     const QString type = sipCpp->type();
 
-    if ( type == QStringLiteral( "singleSymbol" ) )
+    if ( type == QLatin1String( "singleSymbol" ) )
       sipType = sipType_QgsSingleSymbolRenderer;
-    else if ( type == QStringLiteral( "categorizedSymbol" ) )
+    else if ( type == QLatin1String( "categorizedSymbol" ) )
       sipType = sipType_QgsCategorizedSymbolRenderer;
-    else if ( type == QStringLiteral( "graduatedSymbol" ) )
+    else if ( type == QLatin1String( "graduatedSymbol" ) )
       sipType = sipType_QgsGraduatedSymbolRenderer;
-    else if ( type == QStringLiteral( "RuleRenderer" ) )
+    else if ( type == QLatin1String( "RuleRenderer" ) )
       sipType = sipType_QgsRuleBasedRenderer;
-    else if ( type == QStringLiteral( "heatmapRenderer" ) )
+    else if ( type == QLatin1String( "heatmapRenderer" ) )
       sipType = sipType_QgsHeatmapRenderer;
-    else if ( type == QStringLiteral( "invertedPolygonRenderer" ) )
+    else if ( type == QLatin1String( "invertedPolygonRenderer" ) )
       sipType = sipType_QgsInvertedPolygonRenderer;
-    else if ( type == QStringLiteral( "pointCluster" ) )
+    else if ( type == QLatin1String( "pointCluster" ) )
       sipType = sipType_QgsPointClusterRenderer;
-    else if ( type == QStringLiteral( "pointDisplacement" ) )
+    else if ( type == QLatin1String( "pointDisplacement" ) )
       sipType = sipType_QgsPointDisplacementRenderer;
-    else if ( type == QStringLiteral( "25dRenderer" ) )
+    else if ( type == QLatin1String( "25dRenderer" ) )
       sipType = sipType_Qgs25DRenderer;
-    else if ( type == QStringLiteral( "nullSymbol" ) )
+    else if ( type == QLatin1String( "nullSymbol" ) )
       sipType = sipType_QgsNullSymbolRenderer;
     else
       sipType = 0;
@@ -260,7 +271,7 @@ class CORE_EXPORT QgsFeatureRenderer
      *     skip_the_curren_feature()
      * \endcode
      */
-    virtual QgsFeatureRenderer::Capabilities capabilities() { return nullptr; }
+    virtual QgsFeatureRenderer::Capabilities capabilities() { return QgsFeatureRenderer::Capabilities(); }
 
     /**
      * Returns list of symbols used by the renderer.
@@ -498,8 +509,9 @@ class CORE_EXPORT QgsFeatureRenderer
     /**
      * Clones generic renderer data to another renderer.
      * Currently clones
-     *  * Order By
-     *  * Paint Effect
+     *
+     * - Order By
+     * - Paint Effect
      *
      * \param destRenderer destination renderer for copied effect
      */
@@ -507,16 +519,16 @@ class CORE_EXPORT QgsFeatureRenderer
 
     QString mType;
 
-    bool mUsingSymbolLevels;
+    bool mUsingSymbolLevels = false;
 
     //! The current type of editing marker
     int mCurrentVertexMarkerType;
     //! The current size of editing marker
-    double mCurrentVertexMarkerSize;
+    double mCurrentVertexMarkerSize = 2;
 
     QgsPaintEffect *mPaintEffect = nullptr;
 
-    bool mForceRaster;
+    bool mForceRaster = false;
 
     /**
      * \note this function is used to convert old sizeScale expressions to symbol
@@ -532,7 +544,7 @@ class CORE_EXPORT QgsFeatureRenderer
 
     QgsFeatureRequest::OrderBy mOrderBy;
 
-    bool mOrderByEnabled;
+    bool mOrderByEnabled = false;
 
   private:
 #ifdef SIP_RUN
