@@ -101,15 +101,15 @@ class QgsCachedImageFetcher: public QgsImageFetcher
 
 
 /**
-
-  \brief Data provider for OGC WMS layers.
-
-  This provider implements the
-  interface defined in the QgsDataProvider class to provide access to spatial
-  data residing in a OGC Web Map Service.
-
+ *
+ * \brief Data provider for OGC WMS layers.
+ *
+ * This provider implements the
+ * interface defined in the QgsDataProvider class to provide access to spatial
+ * data residing in a OGC Web Map Service.
+ *
 */
-class QgsWmsProvider : public QgsRasterDataProvider
+class QgsWmsProvider final: public QgsRasterDataProvider
 {
     Q_OBJECT
 
@@ -136,6 +136,8 @@ class QgsWmsProvider : public QgsRasterDataProvider
 
     QgsCoordinateReferenceSystem crs() const override;
 
+    QgsRasterDataProvider::ProviderCapabilities providerCapabilities() const override;
+
     /**
      * Reorder the list of WMS layer names to be rendered by this server
      * (in order from bottom to top)
@@ -213,9 +215,9 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QString name() const override;
     static QString providerKey();
     QString description() const override;
-    void reloadData() override;
     bool renderInPreview( const QgsDataProvider::PreviewContext &context ) override;
     QList< double > nativeResolutions() const override;
+    QgsLayerMetadata layerMetadata() const override;
 
     static QVector<QgsWmsSupportedFormat> supportedFormats();
 
@@ -273,7 +275,9 @@ class QgsWmsProvider : public QgsRasterDataProvider
   private:
 
     //! In case of XYZ tile layer, setup capabilities from its URI
-    void setupXyzCapabilities( const QString &uri );
+    void setupXyzCapabilities( const QString &uri, const QgsRectangle &sourceExtent = QgsRectangle(), int sourceMinZoom = -1, int sourceMaxZoom = -1, double sourceTilePixelRatio = 0. );
+    //! In case of MBTiles layer, setup capabilities from its metadata
+    bool setupMBTilesCapabilities( const QString &uri );
 
     QImage *draw( QgsRectangle const   &viewExtent, int pixelWidth, int pixelHeight, QgsRasterBlockFeedback *feedback );
 
@@ -331,7 +335,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QString toParamValue( const QgsRectangle &rect, bool changeXY );
 
     /* \brief add SRS or CRS parameter */
-    void setSRSQueryItem( QUrl &url );
+    void setSRSQueryItem( QUrlQuery &url );
 
     bool ignoreExtents() const override;
 
@@ -342,6 +346,13 @@ class QgsWmsProvider : public QgsRasterDataProvider
     void createTileRequestsWMTS( const QgsWmtsTileMatrix *tm, const QgsWmsProvider::TilePositions &tiles, QgsWmsProvider::TileRequests &requests );
     void createTileRequestsXYZ( const QgsWmtsTileMatrix *tm, const QgsWmsProvider::TilePositions &tiles, QgsWmsProvider::TileRequests &requests );
 
+    /**
+      * Add WMS-T parameters to the \a query, if provider has temporal properties
+      *
+      * \since QGIS 3.14
+      */
+    void addWmstParameters( QUrlQuery &query );
+
     //! Helper structure to store a cached tile image with its rectangle
     typedef struct TileImage
     {
@@ -366,10 +377,10 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QString layerMetadata( QgsWmsLayerProperty &layer );
 
     //! remove query item and replace it with a new value
-    void setQueryItem( QUrl &url, const QString &key, const QString &value );
+    void setQueryItem( QUrlQuery &url, const QString &key, const QString &value );
 
     //! add image FORMAT parameter to url
-    void setFormatQueryItem( QUrl &url );
+    void setFormatQueryItem( QUrlQuery &url );
 
     //! Name of the stored connection
     QString mConnectionName;
@@ -408,6 +419,9 @@ class QgsWmsProvider : public QgsRasterDataProvider
 
     std::unique_ptr<QgsImageFetcher> mLegendGraphicFetcher;
 
+    //! TRUE if an error was encountered while fetching a legend graphic
+    bool mLegendGraphicFetchErrored = false;
+
     /**
      * Visibility status of the given active sublayer
      */
@@ -443,7 +457,6 @@ class QgsWmsProvider : public QgsRasterDataProvider
      */
     QString mError;
 
-
     /**
      * The mime type of the message
      */
@@ -466,6 +479,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QStringList mSupportedGetFeatureFormats;
 
     QgsCoordinateReferenceSystem mCrs;
+    QgsLayerMetadata mLayerMetadata;
 
     //! Parsed response of server's capabilities - initially (or on error) may be invalid
     QgsWmsCapabilities mCaps;
@@ -473,6 +487,9 @@ class QgsWmsProvider : public QgsRasterDataProvider
     //! User's settings (URI, authorization, layer, style, ...)
     QgsWmsSettings mSettings;
 
+    //! Temporal range member
+    QgsDateTimeRange mRange;
+
     QList< double > mNativeResolutions;
 
     friend class TestQgsWmsProvider;
@@ -577,12 +594,14 @@ class QgsWmsStatistics
 
 Q_DECLARE_TYPEINFO( QgsWmsProvider::TilePosition, Q_PRIMITIVE_TYPE );
 
-class QgsWmsProviderMetadata: public QgsProviderMetadata
+class QgsWmsProviderMetadata final: public QgsProviderMetadata
 {
   public:
     QgsWmsProviderMetadata();
-    QgsWmsProvider *createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options ) override;
+    QgsWmsProvider *createProvider( const QString &uri, const QgsDataProvider::ProviderOptions &options, QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() ) override;
     QList<QgsDataItemProvider *> dataItemProviders() const override;
+    QVariantMap decodeUri( const QString &uri ) override;
+    QString encodeUri( const QVariantMap &parts ) override;
 };
 
 #endif
